/// Serialisation mechanism that is chosen over bytestring concatenation
/// both for performance and simplicity.
///
/// See [`Serializable`].
pub trait Serializer {
    /// Writes bytes from a slice. Should advance value of [`Serializer::tell()`] by `buf.len()`.
    fn write(&mut self, buf: &[u8]);
    /// Current position of the stream. It's expected to be used by [`crate::std::inlining`]
    fn tell(&self) -> usize;
}

/// See [`Serializer`].
pub trait Serializable {
    /// Expected to use [`Serializer::write`].
    fn serialize(&self, serializer: &mut dyn Serializer);
}

impl Serializer for Vec<u8> {
    fn write(&mut self, buf: &[u8]) {
        self.extend(buf);
    }

    fn tell(&self) -> usize {
        self.len()
    }
}
